/*
 * (c) 1996, 1997, 1998, 1999 by Polar Pyramid.
 */

#include <assert.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>   // for gettimeofday
#include <sys/time.h> // for struct timeval
#include <time.h>     // for clock

#include "auxil.h"

/*
 * auxil_gettime(void)
 *
 * returnes the nr of seconds since this function was first called.
 */

float auxil_gettime(void) 
{
  static int initialised=0;
  static struct timeval start;
  struct timeval now;
  float deltasec;
  float deltausec;

  if (!initialised)
  {
    gettimeofday(&start,0);
    initialised=1;
    return 0.0;
  }
  gettimeofday(&now,0);
  deltasec  = now.tv_sec - start.tv_sec;
  deltausec = (now.tv_usec - start.tv_usec) * 0.000001;
  return deltasec + deltausec;
}

//
// GetRootPath
//

std::string GetRootPath()
{
  static const char *root = getenv("NBROOT");
  if (!root)
  {
    root = NBROOT;
    printf("Using compiled in NBROOT value %s\n", root);
  }

  return root;
}

//
// GetSoundPath
//

std::string GetSoundPath()
{
  static const char *path = getenv("NBSOUNDPATH");
  if (path)
    return path;

  return GetRootPath() + "/sound/";
}


//
// GetLogPath
//

std::string GetLogPath()
{
  static const char *path = getenv("NBLOGPATH");
  if (path)
    return path;

  return "/tmp/";
}


//
// GetSpritePath
//

std::string GetSpritePath()
{
  static const char *path = getenv("NBSPRITEPATH");
  if (path)
    return path;

  return LIBROOT "/sprite/";
}

//
// GetWorldPath
//

std::string GetWorldPath()
{
  static const char *path = getenv("NBWORLDPATH");
  if (path)
    return path;

  return GetRootPath() + "/world/";
}

//
// GetImagePath
//

std::string GetImagePath()
{
  static const char *path = getenv("NBIMAGEPATH");
  if (path)
    return path;

  return GetRootPath() + "/image/";
}

