/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>

#include "auxil.h"
#include "inpkey.h"


static bool k_l=false, k_r=false, k_u=false, k_d=false;
static bool k_A=false, k_B=false, k_C=false;


InputKeyBoard::InputKeyBoard()
{
  dpy = XOpenDisplay(getenv("DISPLAY"));
  if (!dpy)
  {
    auxil_log(LOG_ERR,"Cannot open X11 Display\n");
    return;
  }
  assert(dpy);
  int retval = XGrabKeyboard
  (
    dpy,
    DefaultRootWindow(dpy), 
    True, 
    GrabModeAsync,
    GrabModeAsync,
    CurrentTime
  );

  axiscount = 2;
  buttoncount = 3;

  opened = true;
}


InputKeyBoard::~InputKeyBoard()
{
  XUngrabKeyboard(dpy, CurrentTime);
  XCloseDisplay(dpy);
}


void InputKeyBoard::Update(float dt)
{

  bool foundone;
  do
  {
    XEvent evt;
    foundone = XCheckMaskEvent(dpy, KeyPressMask | KeyReleaseMask, &evt);
    if (foundone)
    {
      // handle the event
      XKeyEvent keyevt = evt.xkey;
      //printf("type=%d, state=%d, code=%d\n", keyevt.type, keyevt.state, keyevt.keycode);
      assert(keyevt.type==KeyPress || keyevt.type==KeyRelease);

      if (keyevt.keycode==100) k_l = (keyevt.type==KeyPress);
      if (keyevt.keycode==102) k_r = (keyevt.type==KeyPress);
      if (keyevt.keycode== 98) k_u = (keyevt.type==KeyPress);
      if (keyevt.keycode==104) k_d = (keyevt.type==KeyPress);

      if (keyevt.keycode== 65) k_A = (keyevt.type==KeyPress);
      if (keyevt.keycode== 37) k_B = (keyevt.type==KeyPress);
      if (keyevt.keycode==  9) k_C = (keyevt.type==KeyPress);
    }
  } while(foundone);
}


float InputKeyBoard::GetAxisValue(int id) const
{
  assert(id<axiscount);
  if (id==0)
  {
    float retval=0.0;
    if (k_r) retval += 1.0;
    if (k_l) retval -= 1.0;
    return retval;
  }
  if (id==1)
  {
    float retval=0.0;
    if (k_u) retval += 1.0;
    if (k_d) retval -= 1.0;
    return retval;
  }
  return 0.0;
}


bool InputKeyBoard::GetButtonValue(int id) const
{
  assert(id<buttoncount);
  if (id==0)
    return k_A;
  if (id==1)
    return k_B;
  if (id==2)
    return k_C;

  return false;
}

