/*
 * (c) 1996, 1997, 1998 by Polar Pyramid.
 */

// Description:
//	Includes all usefull include files of math
//
// History:
//	October 20, 1996:	creation JSt
//	Februari 28, 1997:	added get_angle_from_2/3d_heading
//      April 5, 1997:          RotateVector3WithRealAngle


#ifndef MATHS_HH
#define MATHS_HH

#include "config.h"

#include "const.h"
#include "utils.h"

#include "vector2.h"
#include "vector3.h"
#include "vector3.inl"
#include "vector2.inl"
#include "sintab.h"



inline void RotateVector2(int    r,
                                 const Vector2 &v,
                                 Vector2       &resv)
{
  resv.x = v.x*D_COS(r) - v.y*D_SIN(r);
  resv.y = v.x*D_SIN(r) + v.y*D_COS(r);
}

inline void ValidateAngle(float &angle)
{
  while (angle < 0) // looped at most twice so faster than mod ?
    angle += TWEEPI; 
  while (angle > TWEEPI)
    angle -= TWEEPI;
}

inline void RotateVector2WithAngle(float         angle, 
					  const Vector2 &v,
					  Vector2       &resv)
{
  unsigned char r = (unsigned char) (angle*(SINTABSIZE/(2*EENPI)));
  r &= (SINTABSIZE - 1);
  resv.x = v.x*D_COS(r) - v.y*D_SIN(r);
  resv.y = v.x*D_SIN(r) + v.y*D_COS(r);
}

inline void RotateVector2WithRealAngle(float         angle, 
					      const Vector2 &v,
					      Vector2       &resv)
{
  float cs = cos(angle);
  float sn = sin(angle);
  resv.x = v.x*cs - v.y*sn;
  resv.y = v.x*sn + v.y*cs;
}

//
// Rotate around x-axis
//


inline Vector2 get_2d_heading_from_angle(float angle)
{
  unsigned char r = (unsigned char) (angle*(SINTABSIZE/(2*EENPI)));
  r &= (SINTABSIZE - 1);
  return Vector2(D_COS(r), -D_SIN(r));
}

inline float get_angle_from_2d_heading(const Vector2 &heading)
{
  if (heading.y < 0.0)
    if (heading.x > 0.0)
      return Asin(-heading.y);          // NE
    else
      return EENPI - Asin(-heading.y);     // NW
  else
    if (heading.x > 0.0)
      return TWEEPI - Asin(heading.y);  // SE
    else
      return EENPI + Asin(heading.y);      // SW
}


#endif
