/*
 * (c) 1996, 1997, 1998 by Polar Pyramid.
 */

// Description:
//	The Vector2 class is an array that contains 2 floats
//
//      Examples of usage:
//
//        Vector2 Vector2A(20, 33);
//        Vector2 Vector2B(-10, 3);
//        cout << Vector2A << " + " << Vector2B<< " = ";
//        cout << (Vector2A + Vector2B) << endl;
//      
//
//
// History:
//	Oct 31, 1996:	creation JSt
//                      operators are defined as member functions, so they
//                      can be inherited.

#ifndef VECTOR2_HH
#define VECTOR2_HH

class Vector3;

class Vector2         // float array with length 2
{
public:
  // the array data 

  float           x;
  float           y;

  // constructors

  Vector2();
  Vector2(const Vector3 &);
  Vector2(float x, float y);

  // array access

  float& operator [] (int);
  inline const float& operator [] (int) const;

  // assignment

  Vector2& assign(float x, float y);

#ifdef WHY_IMPLEMENT_BUILDINS
  // assignment operator slows things down considerably !!
  // e.g. a = b + c; is then much slower, so we do not define this operator

  Vector2(const Vector2 &);
  Vector2& operator =(const Vector2& v);
#endif

  // math operators 

  Vector2 operator - () const;
  Vector2& operator += (const Vector2&);
  Vector2& operator -= (const Vector2&);
  Vector2& operator *= (float);
  Vector2& operator /= (float);

  Vector2 operator + (const Vector2&) const;
  Vector2 operator - (const Vector2&) const;
  Vector2 operator / (float) const;

  bool operator == (const Vector2&) const;
  bool operator != (const Vector2&) const;

  // conversion 

  void copyTo(float [2]) const;
  void Dump(void) const;

  // friends

  friend Vector2 operator*(float c, Vector2 const &v);
  friend Vector2 operator*(Vector2 const &v, float c);

  // vector2 stuff
  // operations

  double       length(void) const;
  inline float lengthSqr(void) const;
  void         normalize(void);

  // dot product

  inline float dot(const Vector2&) const;

  // point
  // distance

  float distanceSqr(const Vector2&) const;
  double distance(const Vector2&) const;

  inline void Snap(float grid_spacing);
};

typedef class Vector2 *Vector2Ptr;

#endif
