/*
 * (c) 1996, 1997, 1998 by Polar Pyramid.
 */

// Description:
//	The Vector3 class is an array that contains 3 floats
//
//      Examples of usage:
//
//        Vector3 Vector3A(20, 33, 1);
//        Vector3 Vector3B(-10, 3, 2);
//        cout << Vector3A << " + " << Vector3B<< " = ";
//        cout << (Vector3A + Vector3B) << endl;
//      
//
//
// History:
//	Oct 31, 1996:	creation JSt
//                      operators are defined as member functions, so they
//                      can be inherited.
//

#ifndef VECTOR3_HH
#define VECTOR3_HH

class Vector2;

class Vector3         // float array with length 3
{
public:
  // the array data 

  float           x;
  float           y;
  float           z;

  // constructors

  Vector3();
  Vector3(float x, float y, float z);

  // array access

  float& operator [] (int);
  inline const float& operator [] (int) const;

  // assignment

  Vector3& assign(float x, float y, float z);

#ifdef WHY_IMPLEMENT_BUILDINS
  // assignment operator slows things down considerably !!
  // e.g. a = b + c; is then much slower, so we do not define this operator

  Vector3(const Vector3 &);
  Vector3& operator =(const Vector3& v);
#endif

  // math operators 

  Vector3 operator - () const;
  Vector3& operator += (const Vector3&);
  Vector3& operator += (const Vector2&);
  Vector3& operator -= (const Vector3&);
  Vector3& operator -= (const Vector2&);
  Vector3& operator *= (float);
  Vector3& operator /= (float);

  Vector3 operator + (const Vector3&) const;
  Vector3 operator + (const Vector2&) const;
  Vector3 operator - (const Vector3&) const;
  Vector3 operator - (const Vector2&) const;
  Vector3 operator / (float) const;

  bool operator == (const Vector3&) const;
  bool operator != (const Vector3&) const;

  // conversion 

  void copyTo(float [3]) const;
  void Dump(void) const;

  // friends

  friend Vector3 operator*(float c, Vector3 const &v);
  friend Vector3 operator*(Vector3 const &v, float c);

  // vector3 stuff
  // operations

  double        length(void) const;
  inline float  lengthSqr(void) const;
  double        length2d(void) const;
  inline float  lengthSqr2d(void) const;
  void          normalize(void);

  int dominant_axis(void);

  // conversions

  Vector2 ToVector2(void) const;

  // cross product

  Vector3 operator * (const Vector3&) const;

  // dot product

  inline float dot(const Vector3&) const;
  inline float dot2d(const Vector3&) const;

  // point
  // distance

  float distanceSqr(const Vector3&) const;
  double distance(const Vector3&) const;
  float distanceSqr2d(const Vector3&) const;
  double distance2d(const Vector3&) const;

  void UpdateMin(const Vector3 &vertex);
  void UpdateMax(const Vector3 &vertex);

  inline bool IsNormalized(void);

  inline void Snap(float grid_spacing);

  inline void SanityCheck(void) const;
};
typedef class Vector3 *Vector3Ptr;

#endif
