/*
 * (c) 1996, 1997, 1998 by Polar Pyramid.
 */

// Description:
//	Inline member functions of the Vector3 class
//
// History:
//	Oct 31, 1996:	creation JSt
//
//

#ifndef VECTOR3_INL
#define VECTOR3_INL

#include <assert.h>

#ifdef sgi
#include <ieeefp.h>
#endif

#include "auxil.h"

#include "vector2.h"
#include "vector3.h"

/////////////////////////////////////////////////////////////////////////////
//
// Default Constructor
//

inline Vector3::Vector3()
 : x(0), y(0), z(0)
{
}

/////////////////////////////////////////////////////////////////////////////
//
// Copy Constructor
//

#ifdef WHY_IMPLEMENT_BUILDINS
inline Vector3::Vector3(const Vector3& v)
  : x(v.x), y(v.y), z(v.z)
{
}
#endif

/////////////////////////////////////////////////////////////////////////////
//
// Element Constructor
//

inline Vector3::Vector3(float ix, float iy, float iz)
  : x(ix), y(iy), z(iz)
{
}

/////////////////////////////////////////////////////////////////////////////
//
// Array Access
//

inline float& Vector3::operator [](int i)
{
  assert(i >= 0 && i <= 2);
  switch (i)	
  {
  case 0: return x;
  case 1: return y;
  case 2: return z;
  }
  ASSERT(false);
  return x;
}

inline const float& Vector3::operator [](int i) const
{
  assert(i >= 0 && i <= 2);
  switch (i)	
  {
  case 0: return x;
  case 1: return y;
  case 2: return z;
  }
  ASSERT(false);
  return x;
}

/////////////////////////////////////////////////////////////////////////////
//
// Assignment
//

inline Vector3& Vector3::assign(float ix, float iy, float iz)
{
  x = ix;
  y = iy;
  z = iz;
  return *this;
}

#ifdef WHY_IMPLEMENT_BUILDINS
// assignment operator slows things down considerably !!
// e.g. a = b + c; is then much slower, so we do not define this operator
/////////////////////////////////////////////////////////////////////////////
//
// Vector3 = Vector3
//

inline Vector3& Vector3::operator =(const Vector3& v)
{
  x = v.x; 
  y = v.y;
  z = v.z;
  return *this;
}
#endif

/////////////////////////////////////////////////////////////////////////////
// Math Operators
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
// - Vector3
//

inline Vector3 Vector3::operator -() const
{
  return Vector3(-x, -y, -z);
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 += Vector3
//

inline Vector3& Vector3::operator += (const Vector3& v)
{
  x += v.x; 
  y += v.y;
  z += v.z;
  return *this;
}


/////////////////////////////////////////////////////////////////////////////
//
// Vector3 -= Vector3
//

inline Vector3& Vector3::operator -= (const Vector3& v)
{
  x -= v.x; 
  y -= v.y;
  z -= v.z;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 += Vector2
//

inline Vector3& Vector3::operator += (const Vector2& v)
{
  x += v.x; 
  y += v.y;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 -= Vector2
//

inline Vector3& Vector3::operator -= (const Vector2& v)
{
  x -= v.x; 
  y -= v.y;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 *= float
//


inline Vector3& Vector3::operator *= (float c)
{
  x *= c; 
  y *= c;
  z *= c;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 /= float
//


inline Vector3& Vector3::operator /= (float c)
{
  float inv = 1.0f/c;
  x *= inv; 
  y *= inv;
  z *= inv;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 + Vector3
//


inline Vector3 Vector3::operator + (const Vector3& v) const
{
  return Vector3(x + v.x, 
		 y + v.y,
		 z + v.z);
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 - Vector3
//


inline Vector3 Vector3::operator - (const Vector3& v) const
{
  return Vector3(x - v.x, 
		 y - v.y,
		 z - v.z);
}


/////////////////////////////////////////////////////////////////////////////
//
// Vector3 + Vector2
//


inline Vector3 Vector3::operator + (const Vector2& v) const
{
  return Vector3(x + v.x, 
		 y + v.y,
		 z);
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 - Vector2
//


inline Vector3 Vector3::operator - (const Vector2& v) const
{
  return Vector3(x - v.x, 
		 y - v.y,
		 z);
}


/////////////////////////////////////////////////////////////////////////////
//
// Vector3 / float
// 


inline Vector3 Vector3::operator / (float c) const
{
  //  assert(!IsZero(c));
  return Vector3(x/c, 
		 y/c,
		 z/c);
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 == Vector3
//


inline bool Vector3::operator == (const Vector3& v) const
{
  return ((x == v.x) && 
          (y == v.y) &&
          (z == v.z));
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 != Vector3
//


inline bool Vector3::operator != (const Vector3& v) const
{
  return (!(*this == v));
}

/////////////////////////////////////////////////////////////////////////////
//
// float * Vector3
//


inline Vector3 operator * (float c, Vector3 const &v)
{
  return Vector3(v.x*c, 
		 v.y*c,
		 v.z*c);
}

/////////////////////////////////////////////////////////////////////////////
//
// Vector3 * float
//


inline Vector3 operator * (Vector3 const &v, float c)
{
  return Vector3(v.x*c, 
		 v.y*c,
		 v.z*c);
}

/////////////////////////////////////////////////////////////////////////////
//
// copyTo
//

inline void Vector3::copyTo(float f[3]) const
{
  f[0] = x; 
  f[1] = y;
  f[2] = z;
}

inline void Vector3::Dump(void) const
{
  auxil_log(LOG_INF, "[%g %g %g]", x, y, z);
}

/////////////////////////////////////////////////////////////////////////////
//
// length : returns the length of the vector3 as a double, note that 
//          this has to return a double because it uses the sqrt function
//

inline double Vector3::length() const
{
  return sqrt(lengthSqr());
}

/////////////////////////////////////////////////////////////////////////////
//
// lengthSqr : returns the squared length of the vector3
//


inline float Vector3::lengthSqr() const
{
  return x*x + y*y + z*z;
}

/////////////////////////////////////////////////////////////////////////////
//
// length2d : returns the length of the vector3 as a double, note that 
//          this has to return a double because it uses the sqrt function
//


inline double Vector3::length2d() const
{
  return sqrt(lengthSqr2d());
}

/////////////////////////////////////////////////////////////////////////////
//
// lengthSqr3d : returns the squared 2d length of the vector3
//

inline float Vector3::lengthSqr2d() const
{
  return x*x + y*y;
}

/////////////////////////////////////////////////////////////////////////////
//
// normalize : will normalize the vector3, make it of length 1
//

inline void Vector3::normalize(void)
{
  double len = length();
  if (len != 0.0)
    *this /= ((float) len);      // avoid SIGFPE
}

//
// ToVector2
//

inline Vector2 Vector3::ToVector2(void) const
{
  return Vector2(x, y);
}


//
// dominant_axis
//  return the dominant axis index (x=0, y=1, z=2)
//

inline int Vector3::dominant_axis(void)
{
  if (Abs(x) > Abs(y))
  {
    if (Abs(x) > Abs(z))
      return 0;
    else
      return 2;
  }
  else
  {
    if (Abs(y) > Abs(z))
      return 1;
    else
      return 2;
  }
}

/////////////////////////////////////////////////////////////////////////////
//
// dot : returns the dot product of 2 vectors
//

inline float Vector3::dot(const Vector3& v2) const
{
  return x*v2.x + y*v2.y + z*v2.z;
}

/////////////////////////////////////////////////////////////////////////////
//
// dot2d : returns the dot product of 2 vector3s for 2d only
//

inline float Vector3::dot2d(const Vector3& v2) const
{
  return x*v2.x + y*v2.y;
}

/////////////////////////////////////////////////////////////////////////////
//
// * operator : returns the cross product of 2 vector3s
//

inline Vector3 Vector3::operator *(const Vector3& v) const
{
  return Vector3(y*v.z - z*v.y,
				 -x*v.z + z*v.x,
				 x*v.y - y*v.x);
}

/////////////////////////////////////////////////////////////////////////////
//
// distanceSqr : return the squared distance between 2 given points which 
//               avoids taking an expensive sqrt call. Usefull when comparing
//               different distances.
//

inline float Vector3::distanceSqr(const Vector3& v2) const
{
  return Sqr(x - v2.x) +
	Sqr(y - v2.y) +
	Sqr(z - v2.z);
}

/////////////////////////////////////////////////////////////////////////////
//
// distance : return the distance between 2 given points, note that 
//            this has to return a double because it uses the sqrt function
//

inline double Vector3::distance(const Vector3& v2) const
{
  return sqrt(distanceSqr(v2));
}

/////////////////////////////////////////////////////////////////////////////
//
// distanceSqr2d : return the squared distance between 2 given points in 2d 
//                 only (z is ignored)
//

inline float Vector3::distanceSqr2d(const Vector3& v2) const
{
  return Sqr(x - v2.x) +
	 Sqr(y - v2.y);
}

/////////////////////////////////////////////////////////////////////////////
//
// distance : return the distance between 2 given points, note that 
//            this has to return a double because it uses the sqrt function
//

inline double Vector3::distance2d(const Vector3& v2) const
{
  return sqrt(distanceSqr2d(v2));
}

//
// UpdateMin
//

inline void Vector3::UpdateMin(const Vector3 &vertex)
{
  if (vertex.x < x)
    x = vertex.x;
  if (vertex.y < y)
    y = vertex.y;
  if (vertex.z < z)
    z = vertex.z;
}

//
// UpdateMax
//

inline void Vector3::UpdateMax(const Vector3 &vertex)
{
  if (vertex.x > x)
    x = vertex.x;
  if (vertex.y > y)
    y = vertex.y;
  if (vertex.z > z)
    z = vertex.z;
}

inline void Vector3::Snap(float grid_spacing)
{
  x = SnapValue(x, grid_spacing);
  y = SnapValue(y, grid_spacing);
  z = SnapValue(z, grid_spacing);
}


inline bool Vector3::IsNormalized(void)
{
  float lensqr = lengthSqr();
  return (fabs(1.0 - lensqr) < 0.0001);
}


inline void Vector3::SanityCheck(void) const
{
  ASSERT(!isnanf(x));
  ASSERT(!isnanf(y));
  ASSERT(!isnanf(z));
#ifndef sgi
  ASSERT(!isinff(x));
  ASSERT(!isinff(y));
  ASSERT(!isinff(z));
#endif
}

#endif


