/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <typeinfo>

#include "world.h"

#include "soundengine.h"

#include "bullet.h"
#include "enemy.h"
#include "player.h"
#include "colmap.h"
#include "presdb.h"
#include "animpres.h"
#include "generator.h"
#include "shield.h"
#include "scorecounter.h"



#include "collisionhandlers.h"


static bool godmode=getenv("NBGOD");

//
// HELPER FUNCS
//

static void AddExplosion
(
  const Vector3 &pos,
  const Vector2 &dir,
  float vel,
  const World &wld
)
{
  AnimatedPresence *ap = new AnimatedPresence("stofwolkje", "wolk", wld);
  ap->pos = pos;
  ap->dir = dir;
  ap->vel = vel;
  PresenceDB::TheOne().Add(ap);
}


//
// COLLISION FUNCS
//


void BulletEnemyCollision(Bullet &bullet, Enemy &enemy)
{
  enemy.dead = true;
  bullet.dead = true;
 
  AddExplosion(bullet.pos, bullet.dir, 0.2*bullet.vel, enemy.GetWorld());
  AddExplosion(bullet.pos, bullet.dir, 0.4*bullet.vel, enemy.GetWorld());
  SoundEngine::TheOne().Play("explo", 0, 0);

  ScoreCounter::TheOne().Add(1);
}


void BulletPlayerCollision(Bullet &bullet, Player &player)
{
}


void EnemyPlayerCollision(Enemy &enemy, Player &player)
{
  if (player.shielded)
  {
    enemy.dead = true;
    Shield *shield = new Shield("shield", "swirl", player.GetWorld());
    shield->pos = player.GetWorld().GetRandomWorldPosition();
    PresenceDB::TheOne().Add(shield);
    player.shielded = false;
    return;
  }
  if (!godmode && !enemy.dead)
    player.dead=true;
}


void EnemyEnemyCollision(Enemy &e0, Enemy &e1)
{
}


void GeneratorPlayerCollision(Generator &generator, Player &player)
{
  if (player.shielded)
  {
    player.shielded = false;
    generator.dead = true;
    ScoreCounter::TheOne().Add(10);
    // add some explosions
    for (int i=0; i<20; i++)
    {
      Vector2 d
      (
        (rand()&2043)-1023,
        (rand()&2043)-1023
      );
      d.normalize();
      float v = rand()&127;
      AddExplosion(generator.pos, d, v, player.GetWorld());
    }
    SoundEngine::TheOne().Play("explo", 0, 0);
    SoundEngine::TheOne().Play("explo", 0, 20000);
    SoundEngine::TheOne().Play("explo", 0, 40000);
    return;
  }
  if (!godmode && !generator.dead)
    player.dead=true;
}


void ShieldPlayerCollision(Shield &shield, Player &player)
{
  if (!player.shielded)
  {
    player.shielded=true;
    shield.dead=true;
  }
}


void LoadStandardCollisionHandlers(void)
{
  CollideMap &cm = CollideMap::TheOne();

  cm.Add(typeid(Bullet),   typeid(Enemy), (collidefunc_t) BulletEnemyCollision);
  cm.Add(typeid(Bullet),   typeid(Player),(collidefunc_t) BulletPlayerCollision);
  cm.Add(typeid(Enemy),    typeid(Player),(collidefunc_t) EnemyPlayerCollision);
  cm.Add(typeid(Enemy),    typeid(Enemy), (collidefunc_t) EnemyEnemyCollision);
  cm.Add(typeid(Generator),typeid(Player),(collidefunc_t) GeneratorPlayerCollision);
  cm.Add(typeid(Shield),   typeid(Player),(collidefunc_t) ShieldPlayerCollision);
}

