/*
 * Purpose:
 *   Implements CollisionMap
 *   See "More efficient C++, ITEM 31" about Double Dispatching.
 *
 *   When two presences collide, simply call:
 *
 *     CollideMap::TheOne().Collide(p1, p2);
 *
 *   NOTE: Collisions between derivatives of Foo and Bar are not
 *   yet supported. TODO!
 *   Collisions between Bar and Foo (as opposed to Foo and Bar) *is*
 *   supported.
 *
 * (c) 1996, 1997, 1998, 1999 by Polar Pyramid.
 */

#include "presence.h"

#include "colmap.h"

CollideMap::CollideMap()
{
}


CollideMap::~CollideMap()
{
}


void CollideMap::Add(const std::type_info &t0, const std::type_info &t1, collidefunc_t f)
{
  bool dummy;
  assert(LookUp(t0,t1,dummy) == 0);
  typepair_t p(t0.name(),t1.name());
  collide_map[p] = f;
  auxil_log(LOG_INF,"Registered collision funcs for %s vs %s", t0.name(), t1.name());
}


collidefunc_t CollideMap::LookUp
(
  const std::type_info &t0,
  const std::type_info &t1,
  bool  &reversed
) const
{
  typepair_t p0(t0.name(),t1.name());
  collide_map_t::const_iterator ip;
  ip = collide_map.find(p0);
  if (ip != collide_map.end())
  {
    reversed = false;
    return (*ip).second;
  }

  typepair_t p1(t1.name(),t0.name());
  ip = collide_map.find(p1);
  if (ip != collide_map.end())
  {
    reversed = true;
    return (*ip).second;
  }

  return 0;
}


void CollideMap::Collide(Presence &p0, Presence &p1) const
{
  bool reversed;
  collidefunc_t f = LookUp(typeid(p0), typeid(p1), reversed);

  if (!f)
  {
//    auxil_log(LOG_INF, "No collision func for %s vs %s (mapsize %d)", typeid(p0).name(), typeid(p1).name(), collide_map.size());
    return;
  }
  
  if (reversed)
    f(p1,p0);
  else
    f(p0,p1);
}


