/*
 * Purpose:
 *   Implements CollisionMap
 *   See "More efficient C++, ITEM 31" about Double Dispatching.
 *
 *   When two presences collide, simply call:
 *
 *     CollideMap::TheOne().Collide(p1, p2);
 *
 *   NOTE: Collisions between derivatives of Foo and Bar are not
 *   yet supported. TODO!
 *   Collisions between Bar and Foo (as opposed to Foo and Bar) *is*
 *   supported.
 *
 * (c) 1996, 1997, 1998, 1999 by Polar Pyramid.
 */

#ifndef COLMAP_HH
#define COLMAP_HH

#include <string>
#include <typeinfo>
#include <map>

class Presence;

typedef std::pair<std::string, std::string> typepair_t;

typedef void (* collidefunc_t)(Presence &p1, Presence &p2);

typedef std::map<typepair_t, collidefunc_t> collide_map_t;


class CollideMap
{
  public:
    CollideMap();
    ~CollideMap();

    void Add(const std::type_info &t0, const std::type_info &t1, collidefunc_t f);

    collidefunc_t LookUp
    (
      const std::type_info &t0,
      const std::type_info &t1,
      bool  &reversed
    ) const;

    void Collide(Presence &p1, Presence &p2) const;

    static CollideMap &TheOne(void)
    {
      static CollideMap *m = new CollideMap();
      return *m;
    }

  protected:
    collide_map_t collide_map;
};

#endif

