/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include "gfxscreen.h"
#include "spritedb.h"
#include "enemy.h"
#include "presdb.h"
#include "animpres.h"

#include "generator.h"


Generator::Generator
(
  const std::string &name,
  const std::string &spritenm, 
  const std::string &enemyspritenm, 
  const World &w
) :
  Presence(name, spritenm, w),
  enemyspritename(enemyspritenm),
  balloonname("")
{
}


Generator::~Generator()
{
}


void Generator::Spawn(void)
{
  Enemy *enemy = new Enemy("enemy", enemyspritename, theworld);
  enemy->pos = pos;
  PresenceDB::TheOne().Add(enemy);
}


void Generator::Speak(void)
{
}


void Generator::Update(float dt)
{
  Presence::Update(dt); // call parent class update

  // random angular vel change
  if (!(rand()&15))
  {
    switch(rand()&15)
    {
      case 0:
        angularvel =  (rand()&31)/1000.0;
        break;
      case 1:
        angularvel = -(rand()&31)/1000.0;
        break;
      case 2:
        vel = 0.0;
      case 3:
        acceleration = (rand()&7);
      case 4:
        Spawn();
        break;
      case 5:
      case 6:
        Speak();
        break;
      default:
        angularvel = 0.0;
        break;
    }
  }
  if (vel > 50.0)
    acceleration = -4.0;
}


void Generator::SetBalloonName(std::string nm)
{
  balloonname = nm;
}


void Generator::Render(const GfxScreen &scn) const
{
  Presence::Render(scn);

  const Sprite &spr = SpriteDB::TheOne().Get(balloonname);
  Vector2 d = GfxScreen::TheOne().VectorToScreenSpace(Vector3(50,-50,0));
  Vector3 p = pos;
  p[0] += d[0];
  p[1] += d[1];

  scn.DrawSprite(spr, p, false, 0.0);
}

