/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <typeinfo>

#include "gfxscreen.h"

#include "spritedb.h"

#include "player.h"


Player::Player
(
  const std::string &name,
  const std::string &spritenm, 
  const World &w
) :
  Presence(name, spritenm, w),
  shielded(false),
  lifetime(0.0),
  walktime(0.0)
{
  auxil_log
  (
    LOG_INF,
    "Created a '%s' object",
    typeid(Player).name()
  );
}


Player::~Player()
{
}


void Player::Update(float dt)
{
  Presence::Update(dt); // call parent class update
  walktime += (vel/200.0)*dt;
  lifetime += dt;
}


void Player::Render(const GfxScreen &scn) const
{
  const Sprite &spr = SpriteDB::TheOne().Get(spritename);
  float cycletime=0.4;
  scn.DrawSprite(spr, pos, false, fmodf(walktime,cycletime)/cycletime);
  if (shielded)
  {
    const Sprite &shieldspr = SpriteDB::TheOne().Get("swirl");
    scn.DrawSprite(shieldspr, pos, false, 1.0-fmodf(lifetime,0.3)/0.3);
  }
}

