/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <list>


#include "gfxscreen.h"

#include "presence.h"
#include "colmap.h"

#include "presdb.h"

#include "collisionhandlers.h"


PresenceDB::PresenceDB()
{
  LoadStandardCollisionHandlers();
}


PresenceDB::~PresenceDB()
{
}


void PresenceDB::Add(Presence *p)
{
  presences.push_back(p);
}


PresenceDB &PresenceDB::TheOne(void)
{
  static PresenceDB *db = new PresenceDB();
  return *db;
}


void PresenceDB::Render(const GfxScreen &scn) const
{
  std::list<Presence *>::const_iterator ip;
  for (ip=presences.begin(); ip!=presences.end(); ++ip)
    (*ip)->Render(scn);
}


void PresenceDB::Update(float dt)
{
  std::list<Presence *>::iterator ip;
  for (ip=presences.begin(); ip!=presences.end(); ++ip)
    (*ip)->Update(dt);

  // Check for collisions
  CollideMap &cm = CollideMap::TheOne();

  std::list<Presence *>::iterator ip0;
  for (ip0=presences.begin(); ip0!=presences.end(); ++ip0)
  {
    std::list<Presence *>::iterator ip1=ip0;
    ip1++;
    for (; ip1!=presences.end(); ++ip1)
    {
      Presence *p0 = *ip0;
      Presence *p1 = *ip1;
      Vector3 d = p0->pos - p1->pos;
      if (d.length() < (p0->radius + p1->radius))
        cm.Collide(*p0, *p1);
    }
  }

  // Remove all dead presences

  std::list<std::list<Presence *>::iterator> removal_list;
  for (ip=presences.begin(); ip!=presences.end(); ++ip)
    if ((*ip)->dead)
      removal_list.push_back(ip);

  std::list<std::list<Presence *>::iterator>::reverse_iterator rip;
  for (rip=removal_list.rbegin(); rip!=removal_list.rend(); ++rip)
  {
    Presence *deadpresence = *(*rip);
    presences.erase(*rip);
    if (deadpresence->name != "player")
      delete deadpresence;
  }
}


void PresenceDB::Clear(void)
{
  presences.clear();
}


int PresenceDB::Size(void) const
{
  return presences.size();
}


int PresenceDB::Count(const std::type_info &ti) const
{
  int retval=0;

  std::list<Presence *>::const_iterator ip;
  for (ip=presences.begin(); ip!=presences.end(); ++ip)
  {
    if (typeid(*(*ip)) == ti)
      retval++;
  }
  return retval;
}

