/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <typeinfo>	// for typeid()

#include "maths.h"

#include "world.h"
#include "gfxscreen.h"

#include "presence.h"

#include "spritedb.h"


Presence::Presence
(
  const std::string &myname, 
  const std::string &spritenm, 
  const World &w
) :
  name(myname),
  spritename(spritenm),
  pos(0,0,0),
  dir(0,-1),
  ori(0,-1),
  vel(0.0),
  angularvel(0.0),
  acceleration(0.0),
  maxvel(400),
  dead(false),
  radius(15.0),
  wallcollision(false),
  autoorient(true),
  insubstantial(false),
  last_known_good_pos(0,0,0),
  theworld(w)
{
}



Presence::~Presence()
{
}


Vector2 Presence::GetVelocityVector(void) const
{
  return vel * dir;
}


void Presence::Render(const GfxScreen &scn) const
{
  float angle = get_angle_from_2d_heading(ori);
  const Sprite &spr = SpriteDB::TheOne().Get(spritename);

  scn.DrawSprite(spr, pos, true, angle);
}


void Presence::Update(float dt)
{
  // Update orientation
  if (angularvel != 0.0)
  {
    Vector2 newori;
    RotateVector2WithRealAngle(angularvel, ori, newori);
    newori.normalize();
    ori = newori;
  }
  if (autoorient)
    dir = ori;

  // Update velocity of presence
  vel += acceleration;
  if (vel<0.0) vel=0;
  if (vel>maxvel) vel = maxvel;

  // Update position of presence
  Vector3 newpos;
  Vector2 deltap = dt * vel * dir;
  newpos = pos + deltap;

  if (insubstantial || theworld.IsValidWorldPosition(newpos))
  {
    pos = newpos;
    wallcollision = false;
  }
  else
  {
    wallcollision = true;
  }
}

