/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include "setup.h"
#include "gfxscreen.h"
#include "spritedb.h"
#include "sprite.h"

#include "soundengine.h"

#include "scorecounter.h"


ScoreCounter::ScoreCounter() :
  ist_wert(0),
  soll_wert(0),
  speed(3),
  sprite(SpriteDB::TheOne().Get("counter"))
{
}


ScoreCounter::~ScoreCounter()
{
}


void ScoreCounter::RenderDigit(int x, int y, float digit, const GfxScreen &scn)
{
  assert(digit>=0);
  assert(digit<10);
  Vector3 pos(x,y,0);
  scn.DrawScreenSpaceSprite(sprite, pos, false, digit/10.0);
}


void ScoreCounter::Render(const GfxScreen &scn)
{
  int x=PSCREENW-(14+4);
  int y=PSCREENH-(50+4);
  float v=ist_wert;

  for (int digit=0; digit<4; digit++)
  {
    float digitval = fmodf(v, 10.0);
    RenderDigit(x, y, digitval, scn);
    v = v - digitval;
    v = v / 10.0;
    x = x - 14;
  }
}


void ScoreCounter::Reset(void)
{
  soll_wert = 0;
  ist_wert = 0;
}


void ScoreCounter::Set(int v)
{
  soll_wert = v;
}


void ScoreCounter::Add(int v)
{
  soll_wert += v;
}


void ScoreCounter::Update(float dt)
{
  assert(ist_wert <= soll_wert);
  if (ist_wert != soll_wert)
  {
    float delta = soll_wert - ist_wert;
    float change = std::min(delta, dt*speed);

    if ((int)(ist_wert+change) != (int)(ist_wert))
      SoundEngine::TheOne().Play("tick",3,0);

    ist_wert += change;
  }
}


ScoreCounter &ScoreCounter::TheOne(void)
{
  static ScoreCounter *c = new ScoreCounter();
  return *c;
}

