/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <stdio.h>	// for printf()
#include <fcntl.h>	// for open()
#include <errno.h>	// for perror()
#include <assert.h>	// for assert()
#include <stdlib.h>	// for exit()
#include <unistd.h> 	// for close()
#include <sys/ioctl.h>	// for ioctl()
#include <sys/mman.h>	// for mmap()
#include <string.h>

#include "auxil.h"
#include "im32.h"
#include "framebuffer32.h"


FrameBuffer32::FrameBuffer32() :
  device(0),
  w(0),
  h(0),
  linesize(0),
  data(0)
{
  device = open("/dev/fb0", O_RDWR);
  if (device == -1)
  {
    perror("open() failed on framebuffer");
    exit(1);
  }

  int retval = ioctl(device, FBIOGET_VSCREENINFO, &var_info);
  if (retval==-1)
  {
    perror("ioctl(FBIOGET_VSCREENINFO) failed");
    exit(1);
  }

  w=var_info.xres;
  h=var_info.yres;
  auxil_log(LOG_INF,"framebuffer resolution = %dx%d", w, h);

  bpp = var_info.bits_per_pixel;
  if (bpp != 32)
  {
    bpp=32;
    var_info.bits_per_pixel=32;
    retval = ioctl(device, FBIOPUT_VSCREENINFO, &var_info);
    if (retval==-1)
    {
      perror("ioctl(FBIOPUT_VSCREENINFO) failed");
      exit(1);
    }
  }

  assert(bpp==32);
  linesize = w*bpp/8;
  buffersize = linesize * h;

  data = (int *) mmap
  (
    0,
    buffersize,
    PROT_READ | PROT_WRITE,
    MAP_SHARED,
    device,
    0
  );

  if (data == MAP_FAILED)
  {
    perror("mmap() failed");
    exit(1);
  }
  Clear();
}


FrameBuffer32::~FrameBuffer32()
{
  munmap(data, buffersize);
  close(device);
  auxil_log(LOG_INF,"FrameBuffer32 destructed");
}


void FrameBuffer32::ZoomBlit(const Image32 &im32, int xoff, int yoff)
{
  int *reader0 = (int *)im32.data;
  int *reader1 = (int *)im32.data;

  int *writer = (int *)(data + xoff + yoff*w);
  for (int y=0; y<im32.h; y++)
  {
    int *writer0 = writer + (y*2+0)*w;
    int *writer1 = writer + (y*2+1)*w;
    for (int x=0; x<im32.w; x++)
    {
      *writer0++ = *reader0;
      *writer0++ = *reader0;
      reader0++;
    }
#if 0
    for (int x=0; x<im32.w; x++)
    {
      *writer1++ = *reader1;
      *writer1++ = *reader1;
      reader1++;
    }
#endif
  }
}


void FrameBuffer32::Blit(const Image32 &im32, int xoff, int yoff, bool interlace)
{
  static int interlacecounter=0;
  interlacecounter = (interlacecounter+1)&1;

  int *reader = (int *)im32.data;

  int *writer = (int *)(data + xoff + yoff*w);
  for (int y=0; y<im32.h; y++)
  {
    if (!interlace || (interlacecounter == (y&1)))
      memcpy(writer+y*w, reader, im32.w*sizeof(int));
    reader += im32.w;
  }
}


void FrameBuffer32::Clear(void)
{
  memset(data,0,buffersize);
}
