/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/extensions/xf86dga.h>

#include "auxil.h"
#include "framebufferdga.h"
#include "im16.h"
#include "im32.h"


FrameBufferDGA::FrameBufferDGA()
{
  dpy = XOpenDisplay(getenv("DISPLAY"));
  assert(dpy);
  scn = DefaultScreen(dpy);
  bpp = DisplayPlanes(dpy,scn);
  auxil_log(LOG_INF,"bpp is %d", bpp);
  if (bpp!=16 && bpp!=24 && bpp!=32)
    auxil_log(LOG_FAT,"Only 16,24 or 32 bits per pixel is supported.");
  int event_base, error_base;
  int rv = XF86DGAQueryExtension(dpy, &event_base, &error_base);
  assert(rv);
  XF86DGAGetViewPortSize(dpy, scn, &width, &height);
  // get video-ram address, pixel-per-line, bank-size and total amount of video memory
  XF86DGAGetVideo(dpy, scn, &address, &pixel_per_line, &bank_size, &ram);
  auxil_log(LOG_INF,"%dx%d at %p", width, height, address);
  auxil_log(LOG_INF,"pixel_per_line is %d", pixel_per_line);
  auxil_log(LOG_INF,"bank_size is %d", bank_size);
  auxil_log(LOG_INF,"ram is %d", ram);
  if (ram*1024 != bank_size)
    auxil_log(LOG_FAT,"Error - You have an old graphics card that uses banked memory. This is not supported.\n");
  rv = XF86DGAForkApp(scn);
  if (rv == -1)
  {
    perror("XF86DGAForkApp failed");
    auxil_log(LOG_FAT,"Cannot use X11 DGA. Try running in framebuffer mode.");
  }

  assert(width>=640);
  assert(height>=675);

  enabled = false;
}


FrameBufferDGA::~FrameBufferDGA()
{
  Enable(false);
  printf("Closing display...\n");
  XCloseDisplay(dpy);
  printf("Display closed\n");
}


void FrameBufferDGA::Clear(void)
{
  if (!IsEnabled()) return;
  memset(address, 0, width * height * ((bpp==24||bpp==32)?4:2));
}


void FrameBufferDGA::Blit(const Image32 &im32, int x, int y, bool interlace)
{
  if (!IsEnabled()) return;

  x += (width-640)/2;
  y += (height-576)/2;

  if (bpp!=24 && bpp!=32)
  {
    Image16 tmp(im32);
    this->Blit(tmp, x, y, interlace);
    return;
  }

  int *buf = (int *)im32.data;
  int *dst = (int*) address;
  dst = dst + y * width + x;
  int *src = buf;
  for (int y=0; y<im32.h; y++)
  {
    memcpy(dst, src, 4*im32.w);
    src += im32.w;
    dst += width;
  }
}


void FrameBufferDGA::Blit(const Image16 &im16, int x, int y, bool interlace)
{
  if (!IsEnabled()) return;

  if (bpp==32 || bpp==24)
  {
    Image32 tmp(im16);
    this->Blit(tmp, x, y, interlace);
    return;
  }

  unsigned short *buf = (unsigned short *)im16.data;
  unsigned short *dst = (unsigned short*) address;
  dst = dst + y * width + x;
  unsigned short *src = buf;
  for (int y=0; y<im16.h; y++)
  {
    memcpy(dst, src, 2*im16.w);
    src += im16.w;
    dst += width;
  }
}


#if 0
void FrameBufferDGA::ZoomedBlit(int xo, int yo, unsigned short *buf, int w, int h, int zoom)
{
  if (!IsEnabled()) return;
  unsigned short *dst = (unsigned short *) address + yo*width + xo;
  unsigned short *src = buf;
  unsigned short *tmp = new unsigned short [w*zoom];
  for (int y=0; y<h; y++)
  {
    unsigned short *writer = tmp;
    for (int x=0; x<w; x++)
    {
      for (int z=0; z<zoom; z++)
        *writer++ = *src;
      src++;
    }
    for (int z=0; z<zoom; z++)
    {
      memcpy(dst, tmp, 2*w*zoom);
      dst += width;
    }
  }
  delete [] tmp;
}
#endif


void FrameBufferDGA::Enable(bool e)
{
  if (e)
  {
    bool ok;
    ok = XF86DGADirectVideo(dpy, scn, XF86DGADirectGraphics);
    assert(ok);
    ok = XF86DGASetViewPort(dpy, scn, 0, 0);
    assert(ok);
  }
  else
  {
    bool ok = XF86DGADirectVideo(dpy, scn, 0);
    assert(ok);
  }
  enabled = e;
}

