/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include "SDL.h"


#include "auxil.h"
#include "framebuffersdl.h"
#include "im16.h"
#include "im32.h"




FrameBufferSDL::FrameBufferSDL()
{
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
  {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);

  screen = SDL_SetVideoMode(640, 576, 32, SDL_HWSURFACE | SDL_ANYFORMAT | SDL_FULLSCREEN);
  if ( screen == NULL ) 
  {
    fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
    exit(1);
  }
  SDL_WM_SetCaption("Tux versus Clippy", 0);

  switch (screen->format->BytesPerPixel)
  {
    case 1:
      auxil_log
      (
         LOG_FAT,
	 "8 bits per pixel is not supported. Reconfigure your X server to 16/32 bpp.\n"
      );
      break;
    case 2:
      bpp=16;
      break;
    case 3:
      auxil_log
      (
         LOG_FAT,
	 "24 bits per pixel is not supported. Reconfigure your X server to 16/32 bpp.\n"
      );
    case 4:
      bpp=32;
      break;
    default:
      auxil_log
      (
        LOG_FAT,
	"Unknown colordepth of %d bytes per pixel",
        screen->format->BytesPerPixel
      );
      break;
  }
  auxil_log
  (
    LOG_INF,
    "Using SDL colordepth of %d bytes per pixel",
    screen->format->BytesPerPixel
  );
}


FrameBufferSDL::~FrameBufferSDL()
{
  auxil_log(LOG_INF,"Shutting down SDL");
  SDL_Quit();
  auxil_log(LOG_INF,"FrameBufferSDL destructed");
}


void FrameBufferSDL::Clear(void)
{
}


void FrameBufferSDL::Blit(const Image32 &im32, int x, int y, bool interlace)
{
  if (screen->format->BytesPerPixel==2)
  {
    Image16 tmp(im32);
    this->Blit(tmp, x, y, interlace);
    return;
  }

  assert(screen->format->BytesPerPixel==4);

  if ( SDL_MUSTLOCK(screen) ) 
  {
    int retval = SDL_LockSurface(screen);
    assert(retval>=0);
  }

  int *dst = (int *)screen->pixels + y*screen->pitch/4 + x;
  int *src = (int *)im32.data;
  for (int row=0; row<im32.h; row++)
  {
    memcpy(dst, src, 4*im32.w);
    dst += screen->pitch/4;
    src += im32.w;
  }

  if ( SDL_MUSTLOCK(screen) ) 
  {
    SDL_UnlockSurface(screen);
  }
  SDL_UpdateRect(screen, x, y, im32.w, im32.h);
}


void FrameBufferSDL::Blit(const Image16 &im16, int x, int y, bool interlace)
{
  if (screen->format->BytesPerPixel==4)
  {
    Image32 tmp(im16);
    this->Blit(tmp, x, y, interlace);
    return;
  }

  assert(screen->format->BytesPerPixel==2);

  if ( SDL_MUSTLOCK(screen) ) 
  {
    int retval = SDL_LockSurface(screen);
    assert(retval>=0);
  }

  short *dst = (short *)screen->pixels + y*screen->pitch/2 + x;
  short *src = (short *)im16.data;
  for (int row=0; row<im16.h; row++)
  {
    memcpy(dst, src, 2*im16.w);
    dst += screen->pitch/2;
    src += im16.w;
  }

  if ( SDL_MUSTLOCK(screen) ) 
  {
    SDL_UnlockSurface(screen);
  }
  SDL_UpdateRect(screen, x, y, im16.w, im16.h);
}



