/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

/*
 * screen.hh
 *
 * 020911 BSt
 *   ported to c++ and xbox
 * 970201 BSt
 *   initial version
 * 970610 BSt
 *   Added DrawScreenSpaceText()
 * 970914 JSt
 *   Added DrawPixel()
 * 971015 JSt
 *   Added real DrawScreenSpaceText(), renamed old
 */

#ifndef GFXSCREEN_HH
#define GFXSCREEN_HH

#include <string>

#include "maths.h"


class Image16;
class Sprite;


class GfxScreen 
{

public:

  GfxScreen(void);

  ~GfxScreen(void);

  const short *GetVisibleSpriteBuffer(void) const
  {
    return visible_sprite_buf; 
  }

  void GetScreenPos
  (
    const Vector3 &world_pos,
    int            screen_pos[2]
  ) const;

  void RotateBuffer(short *rotbuf);

  void DrawScreenSpaceSprite
  (
    const Sprite   &sprite,
    const Vector3  &pos,
    bool           is_oriented,
    float          parm=0.0      // orientation 0..2PI or animparm 0..1
  ) const;

  void DrawSprite
  (
    const Sprite   &sprite,
    const Vector3  &pos,
    bool           is_oriented,
    float          parm=0.0      // orientation 0..2PI or animparm 0..1
  ) const;

  void SetPivot
  (
    const Vector3 &pos,
    float heading
  );

  const Image16 &GetImage(void) const 
  {
    return *spr_im;
  }

  Vector2 VectorToScreenSpace(const Vector2 &v) const;


  static GfxScreen &TheOne(void);

private:
  Image16        *spr_im;
  short          *sprite_buf;		 // sprite buf, clippingarea inclusive
  short          *visible_sprite_buf;    // sprite buf, clippingarea exclusive
  Vector3        pivot_position;	 // position of centre (pivot) obj
  unsigned char  pivot_discreterotation; // orientation of centre (pivot) obj
};


#endif

