/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "im16.h"
#include "im32.h"

#include "colconv.h"


Image16::Image16(const Image32 &im32) :
  w(im32.w),
  h(im32.h),
  data(0)
{
  data = new short[w*h];
  short *writer = data;
  unsigned char  *reader = im32.data;
  for (int y=0; y<h; y++)
  {
    for (int x=0; x<w; x++)
    {
      *writer++ = RGB2COL(reader[0],reader[1],reader[2]);
      reader+=4;
    }
  }
}


Image16::Image16(int ww, int hh) :
  w(ww),
  h(hh),
  data(0)
{
  data = new short[w*h];
}


Image16::~Image16() 
{
   delete data; 
}


void Image16::Write(const char *fname) const
{
  Image32 tmp(*this);
  tmp.Write(fname);
}


void Image16::Crop(Image32 &dst, int xoff, int yoff) const
{
  assert(xoff+dst.w <= w);
  assert(yoff+dst.h <= h);

  unsigned char *writer=dst.data;
  for (int y=0; y<dst.h; y++)
  {
    short *reader=data+w*(y+yoff) + xoff;
    for (int x=0; x<dst.w; x++)
    {
      *writer++ = COL2R((*reader));
      *writer++ = COL2G((*reader));
      *writer++ = COL2B((*reader));
      *writer++ = 255;
      reader++;
    }
  }
}


void Image16::Crop(Image16 &dst, int xoff, int yoff) const
{
  assert(xoff+dst.w <= w);
  assert(yoff+dst.h <= h);

  short *writer=dst.data;
  for (int y=0; y<dst.h; y++)
  {
    short *reader=data+w*(y+yoff) + xoff;
    memcpy(writer,reader,2*dst.w);
    writer += dst.w;
  }
}


short Image16::Pix(int x, int y) const
{
  assert(x<w);
  assert(y<h);
  return data[y*w+x];
}


void Image16::Paste(const Image16 &other, int xoff, int yoff)
{

  assert(xoff+other.w <= w);
  assert(yoff+other.h <= h);
  for (int y=0; y<other.h; y++)
  {
    memcpy(data+(y+yoff)*w+xoff, other.data+y*other.w, other.w*sizeof(short));
  }
}


void Image16::ClipAndPaste(const Image16 &other, int xoff, int yoff)
{
  int srcx0=0;
  int srcy0=0;
  int srcx1=other.w;
  int srcy1=other.h;

  if (xoff >= w) return;
  if (yoff >= h) return;

  if (xoff + srcx1 <= 0) return;
  if (yoff + srcy1 <= 0) return;

  int hix = xoff+srcx1;
  int hiy = yoff+srcy1;

  if (hix > w)
    srcx1 -= (hix-w);

  if (hiy > h)
    srcy1 -= (hiy-h);

  if (xoff + srcx0 < 0)
  {
    srcx0 += (-xoff);
    xoff=0;
  }

  if (yoff + srcy0 < 0)
  {
    srcy0 += (-yoff);
    yoff = 0;
  }

  int sx = srcx1-srcx0;
  int sy = srcy1-srcy0;
  assert(sx>0);
  assert(sy>0);

  short *reader = other.data + srcy0*other.w+srcx0;
  short *writer = data + yoff*w + xoff;

  for (int y=0; y<sy; y++)
  {
    assert(writer+sx <= data+h*w);
    assert(reader+sx <= other.data+other.h*other.w);
    memcpy(writer, reader, sx*sizeof(short));
    writer += w;
    reader += other.w;
  }
}


void Image16::Clear(short c)
{
  for (int i=0; i<w*h; i++)
    data[i]=c;
}

