/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "im8.h"


Image8::Image8(const char *fname) :
  w(0),
  h(0),
  maxval(255),
  data(0)
{
  assert(fname);
  if (strstr(fname,".pgm"))
  {
    ReadPGM(fname);
    return;
  }
  assert(0);
}


void Image8::ReadPGM(const char *fname)
{
  FILE *f=fopen(fname,"rb");
  assert(f);
  int retval = fscanf(f, "P5 %d %d %d ", &w, &h, &maxval);
  assert(retval==3);
  assert(maxval==255);
  data = new unsigned char[w*h];
  retval = fread(data, w, h, f);
  assert(retval == h);
  fclose(f);
}


Image8::Image8(int ww, int hh) :
  w(ww),
  h(hh),
  maxval(255),
  data(0)
{
  data = new unsigned char[4*w*h];
}


void Image8::Write(const char *fname) const
{
  FILE *f=fopen(fname,"wb");
  fprintf(f,"P5\n%d %d\n%d\n",w,h,maxval);
  int retval = fwrite(data, w, h, f);
  assert(retval == h);
  fclose(f);
}


Image8::~Image8() 
{
  delete data; 
}


void Image8::Invert(void)
{
  for (int i=0; i<w*h; i++)
    data[i] = 255-data[i];
}


Image8 *Image8::Smooth(void) const
{
  float filter[5][5] = 
  {
    {0,0,1,0,0},
    {0,1,2,1,0},
    {1,2,3,2,1},
    {0,1,2,1,0},
    {0,0,1,0,0},
  };

  Image8 *retval = new Image8(w,h);
  for (int y=0; y<h; y++)
  {
    for (int x=0; x<w; x++)
    {
      float total_val=0;
      float total_wgt=0;
      for (int yy=0; yy<5; yy++)
      {
        for (int xx=0; xx<5; xx++)
        {
          int xp = x+xx-2;
          int yp = y+yy-2;
          if (xp>=0 && xp<w && yp>=0 && yp<h)
          {
            total_wgt += filter[yy][xx];
            total_val += filter[yy][xx]*PIX(xp,yp);
          }
        }
      }
      retval->data[y*w+x] = (unsigned char) (total_val/total_wgt);
    }
  }
  return retval;
}


