/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <assert.h>
#include <dlfcn.h>	// for dlopen()
#include <stdio.h>

#include "setup.h"
#include "sprite.h"
#include "auxil.h"


Sprite::Sprite(const std::string &n) :
  name(n),
  w(0),
  h(0),
  framecount(0),
  asprite(0),
  dl(0)
{
  std::string libname = GetSpritePath()+"/"+name+".sprite";

  void *dl = dlopen(libname.c_str(), RTLD_NOW);
  if (!dl)
  {
    printf("dlopen() for '%s' failed: %s\n", name.c_str(), dlerror());
  }

  std::string symname = "alphasprite_" + name;
  asprite = (struct alphasprite *) dlsym(dl, symname.c_str());
  const char *err = dlerror();
  if (err)
  {
    printf("Failed to lookup symbol '%s' from dso\n", symname.c_str());
  }
  assert(!err);

  w = asprite->width;
  h = asprite->height;
  framecount = asprite->framecount;

  for (unsigned int i=0; i<framecount; i++)
  {
    char funcname[128];
    sprintf(funcname,"put_%s%d", name.c_str(), i);
    alphafunctype f = (alphafunctype) dlsym(dl, funcname);
    const char *err = dlerror();
    if (err)
    {
      printf("Failed to look up symbol '%s' from dso\n", funcname);
    }
    assert(!err);
    funcs.push_back(f);
  }

  auxil_log
  (
    LOG_INF,
    "Sprite '%s' created with %d frames of [%dx%d]", 
    name.c_str(), framecount,w,h
  );
  assert(w<=SPRW);
  assert(h<=SPRH);
}


Sprite::~Sprite()
{
  if (dl)
    dlclose(dl);
}


void Sprite::Render(unsigned int frm, int x, int y, short *buf) const
{
  assert(frm < framecount);
  assert(frm < funcs.size());
 
  short *dst = buf+y*SPRW+x;

  alphafunctype f = funcs[frm];

  __asm__ __volatile__
  (
    "       pushl %esi\n"
    "       pushl %edi\n"
    "       pushl %ebx\n"
    "       pushl %ecx\n"
    "       pushl %edx\n"
  );

  f
  (
    (unsigned char*) dst, 
    (unsigned char*) gfx_eng_alpha_table
  );

  __asm__ __volatile__
  (
    "       popl %edx\n"
    "       popl %ecx\n"
    "       popl %ebx\n"
    "       popl %edi\n"
    "       popl %esi\n"
  );
}


