/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#include <assert.h>

#include "SDL.h"

#include "auxil.h"
#include "soundclip.h"
#include "soundengine.h"
#include "soundenginesdl.h"


static void mixaudio(void *unused, unsigned char *stream, int len)
{
  if (len < SOUNDPACKETSIZE*sizeof(short))
    return;
  SoundEngine    *se    = & SoundEngine::TheOne();
  SoundEngineSDL *sesdl = dynamic_cast<SoundEngineSDL *>(se);
  assert(sesdl);
  sesdl->Send(stream, len);
}



SoundEngineSDL::SoundEngineSDL()
{
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
  {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
  }
  else
  {
    atexit(SDL_Quit);

    SDL_AudioSpec fmt;

    /* Set 16-bit stereo audio at 22Khz */
    fmt.freq = 44100;
    fmt.format = AUDIO_S16;
    fmt.channels = 2;
    fmt.samples = SOUNDPACKETSIZE/2;
    fmt.callback = mixaudio;
    fmt.userdata = NULL;

    /* Open the audio device and start playing sound! */
    if ( SDL_OpenAudio(&fmt, NULL) < 0 ) 
    {
      fprintf(stderr, "Unable to open audio: %s\n", SDL_GetError());
    }
    else
    {
      opened = true;
      SDL_PauseAudio(0);

      auxil_log(LOG_INF,"SoundEngineSDL created");
      auxil_log(LOG_INF,"SDL audio buffer size is %d", fmt.size);
    }
  }
}


SoundEngineSDL::~SoundEngineSDL()
{
  if (opened)
    SDL_CloseAudio();
  auxil_log(LOG_INF,"SoundEngineSDL destroyed");
}


void SoundEngineSDL::Send(unsigned char *stream, int len)
{
  SDL_LockAudio();

  SoundPacket *packets[SOUNDCHANNELCOUNT];
  int i,j;
  int cnt=0;
  for (i=0; i<SOUNDCHANNELCOUNT; i++)
  {
    packets[i] = channels[i].Pop();
    if (packets[i])
      cnt++;
  }

  SoundPacket out;
  for (j=0; j<SOUNDPACKETSIZE; j++)
  {
    short v=0;
    for (i=0; i<SOUNDCHANNELCOUNT; i++)
      if (packets[i])
        v += packets[i]->data[j];
    out.data[j] = v;
  }

  memcpy(stream, out.data, SOUNDPACKETSIZE*sizeof(short));

  for (i=0; i<SOUNDCHANNELCOUNT; i++)
    delete packets[i];

  SDL_UnlockAudio();
}


void SoundEngineSDL::Play(const std::string &name, int channr, int delay)
{
  if (!opened)
    return;
  assert(channr < SOUNDCHANNELCOUNT);
  SoundClip *clip = SoundClip::Get(name);
  assert(clip);

  SDL_LockAudio();
  channels[channr].Paste(clip, delay);
  SDL_UnlockAudio();
}

