/*
 * Tux vs Clippy
 * (c)2002-2003 by Bram Stolk
 * b.stolk at chello.nl
 * Licensed under GPL
 */

#ifndef WORLD_H
#define WORLD_H

#include <string>

#include "config.h"	// For HAVE_EXT_HASH_MAP

#include "im16.h"
#include "primdb.h"
#include "wldcell.h"

class Vector3;

// GCC uses three different ways to do hash_maps.
// 3.0 and 3.2 use ext/hash_map in different namespaces.
// 2.95 uses hash_map instead of ext/hash_map.
// I have no idea what GCC 3.1 uses, but I can't check, as Debian
// does not seem to feature GCC 3.1 anymore.

#ifdef HAVE_EXT_HASH_MAP
#include <ext/hash_map>
#if __GNUC_MINOR__ == 0
typedef std::hash_map<int, WorldCell*> cellmap_t;
#else
typedef __gnu_cxx::hash_map<int, WorldCell*> cellmap_t;
#endif
#else
#include <hash_map>
typedef std::hash_map<int, WorldCell*> cellmap_t;
#endif


class World
{
  public:
    World(const std::string &tn);
    ~World();
    void CreateWorld(const std::string &mapname);
    const WorldCell &AddCell(int x, int y, const std::string &primname);
    const WorldCell *GetCell(int x, int y) const;
    void ReadMap(const std::string &fname);
    void RenderRotationBuffer(Vector3 &pos);
    const Image16 &GetRotationBuffer(void) const;
    bool IsValidWorldPosition(const Vector3 &pos) const;
    Vector3 GetRandomWorldPosition(void) const;

  protected:
    std::string tilename;
    PrimitiveDatabase prims;
    cellmap_t cells;
    Image16   rot_im;
};

#endif
